package turtle.talk;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
/**
 * <p>Title: Turtle-talk</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Mmorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

/** Support for analysis of Turtle-talk programs.
 *  Supplies a symbol table and a stack for keeping track of types.
 *  <p>State:
 *  <ul>
 *    <li>  typeStack : a stack of integers representing types  </li>
 *    <li>  address : the next unused variable address </li>
 *    <li>  symbolTable: a funtion from strings to SymboTableEntry objects </li>
 *  </ul>
 *  Class invariant: 
 *  <ul>
 *    <li>  0 &lt;= address &lt;= Interpreter.VARMAX  </li>
 *    <li>  symbol table contains no SymbolTableEntries of type UNKNOWN_ENTRY </li>
 *    <li>  for any name in dom(symbolTable) symbolTable[name].getName() == name
 *  </ul>
 *  Collaborations:
 *  <ul>
 *    <li> Has many SymboTableEntry objects in the symbolTable </li>
 *    <li> Uses ContraintError to signal violations of language constraints </li>
 *    <li> Uses ResourceExhausted to signal problems with resource limits </li>
 *  </ul>
 */
class Analyser {

    /** BOOL_TYPE an integer representation of type bool. */
    public static final int BOOL_TYPE = 0 ;
    
    /** INT_TYPE an integer representation of type int. */
    public static final int INT_TYPE = 1 ;
    
    private Map<String, SymbolTableEntry> symbolTable = new HashMap<String, SymbolTableEntry>() ;
    private int address = 0 ;
    
    private List<Integer> typeStack = new ArrayList<Integer>() ;
    
    /** Analyser. Constructs the analyser with the standard definitions.
     *  Standard definitions are for
     *  <ul>
     *    <li> Type int </li>
     *    <li> Type bool </li>
     *    <li> Constant true </li>
     *    <li> Constant false </li>
     *  </ul>
     *  */
    public Analyser() {
        establishStandardContext() ;
    }
    
    /** Check that a constraint is met.
     *  <p>Intended function
     *  <pre>
     *     [ b      --> do nothing
     *     | not b  --> throw a ConstraintError with message str
     *     ]
     *   </pre>
     *  @param b the result of a check.
     *  @param str an error message
     *  @throws ConstraintError if b is false.
     */
    public void check( boolean b, String str ) throws ConstraintError {
        if( !b ) throw new ConstraintError( str ) ; }

    /** Check that the state is as expected when analysis is finished
     *  <p>Intended function
     *  <pre>
     *     [ address=0 and typeStack is empty --> do nothing ]
     *   </pre>
     */
    public void done() {
        Assert.assertTrue( address==0, "not all variable scopes ended.") ;
        Assert.assertTrue( typeStack.size() == 0, "type stack not empty" ) ;
    }
    
    /** Enter a variable into the symbol table
     *  <p>Intended function
     *  <pre>
     *     [ address == Interpreter.VARMAX --> throw ResourceExhausted
     *     | previous entry is not of kind UNKNOWN_ENTRY -->
     *            throw ConstraintError
     *     | type_entry is not of kind TYPE_ENTRY -->
     *            throw ConstraintError
     *     | otherwise -->
     *           symbolTable, address
     *           := symbolTable_( name := varEntry ), address+1
     *           where name is the name from previousEntry
     *           and varEntry is a new entry with fields
     *                         name ~> name
     *                         address ~> address
     *                         type ~> type field of typeEntry
     *                         kind ~> VAR_ENTRY
     *          then return varEntry
     *     ]
     *   </pre>
     *   @param previousEntry the current symbol table entry for the idetifier
     *   @param typeEntry the symbol table entry for the type
     */
    public SymbolTableEntry startVariableScope( SymbolTableEntry previousEntry,
                                                SymbolTableEntry typeEntry )
    throws ResourceExhausted, ConstraintError
    {
        if( address == Interpreter.VARMAX )
            throw new ResourceExhausted("Limit exceeded: Too many variables" ) ;
        check( previousEntry.getKind() == SymbolTableEntry.UNKNOWN_ENTRY,
               "Variable already declared" ) ;
        check( typeEntry.getKind() == SymbolTableEntry.TYPE_ENTRY,
               "Type required" ) ;
        String name = previousEntry.getName() ;
        SymbolTableEntry varEntry = new SymbolTableEntry( SymbolTableEntry.VAR_ENTRY,
                                                          name,
                                                          typeEntry.getType(),
                                                          address ) ;
        address += 1 ;
        symbolTable.put(name, varEntry ) ;
        return varEntry ;
    }
    
    /** Terminate the scope of a variable.
     *  <p>Intended Function:
     *  <pre>
     *  [ the kind of varEntry is VAR_ENTRY and
     *    of the var entries in the symbol table, this is the most recently entered
     *    --> symbolTable, address := symbolTable_name, address - 1
     *        where name is the name of varEntry
     *  ]
     *  </pre>
     *  @param varEntry a variable to be removed from the symbol table.
     */   
    public void endVariableScope( SymbolTableEntry varEntry )
    {
        Assert.assertTrue( varEntry.getKind()== SymbolTableEntry.VAR_ENTRY,
                           "Not variable entry" ) ;
        Assert.assertTrue( varEntry.getAddress() == address-1, "Variables end in wrong order" ) ;
        address -= 1 ;
        String name = varEntry.getName() ;
        symbolTable.remove(name) ;
    }
    
    /** Entry a new entry in the symboltable entry.
     *  <p>Intended Function:
     *  <pre>
     *  [ the kind of newEntry is not UNKNOWN_ENTRY
     *    and kind of newEntry is not VAR_ENTRY -->
     *      symbolTable := symbolTable_( name := newEntry ) 
     *      where name is the name field of the entry
     *  ]
     *  </pre>
     *  @param newEntry a symbol table entry.
     */
    public void enterSymbol( SymbolTableEntry newEntry )
    {
        symbolTable.put(newEntry.getName(), newEntry ) ;
    }
    
    /** Look up an entry in the symbol table.
     *  <p>Intended Function:
     *  <pre>
     *  [ name in dom( symbolTable ) -->
     *       return symbolTable(name)
     *  |
     *    else -->
     *       return an UNKNOWN_ENTRY king symbol table entry with name "name"
     *  ]
     *  </pre>
     *  @param name the name to be looked up.
     */
    public SymbolTableEntry lookup( String name ) {
        if( symbolTable.containsKey( name ) ) {
            return symbolTable.get( name ) ; }
        else {
            return new SymbolTableEntry( SymbolTableEntry.UNKNOWN_ENTRY, name ) ; }
    }
    
    /** Remove the top type from the type stack.
     *  <p>Intended Function:
     *  <pre>
     *  [ type stack is not empty -->
     *       typeStack := typeStack[ 0,.. typeStack.size()-1 ]
     *  |
     *  ]
     *  </pre>
     */
    public void pop() {
        Assert.assertTrue( typeStack.size() != 0, "Type stack empty." ) ;
        typeStack.remove( typeStack.size() - 1 ) ;
    }
    
    /** Add a new top to the type stack
     *  <p>Intended Function:
     *  <pre>
     *  [ typeStack := typeStack ++ [type]
     *  ]
     *  </pre>
     *  @param type the integer representation of the type to add.
     */
    public void push( int type ) {
        typeStack.add( type ); }
    
    
    /** Obtain the top of the type stack
     *  Intended Function:
     *  <pre>
     *  [typeStack is not empty --> return the last element of typeStack
     *  ]
     *  </pre>
     */
    public int top() {
        Assert.assertTrue( typeStack.size() != 0, "Type stack empty." ) ;
        return typeStack.get( typeStack.size() - 1 ) ; }
    
    
    /** Convert a string matching (["0"-"9"])* to an integer.
     *  <p>Intended Function:
     *  <pre>
     *  [ the result fits in an int -->
     *      return the corresponding integer
     *    otherwise --> 
     *      throw a ConstraintError
     *  ]
     *  </pre>
     *  @param intImage the int to convert.
     */   
    static public int parseInt( String intImage ) throws ConstraintError
    {
        try {
            return Integer.parseInt( intImage ) ; }
        catch( NumberFormatException e ) {
            throw new ConstraintError( "int constant is too big" ) ; }
    }
    
    /** Convert the image of a string token to a string.
     *  <p>Intended Function:
     *  <pre>
     *  [ stringImage matches the regular expression for
     *    string tokens ---> return unescape( stringImage[1,..stringImage-1] )
     *  ]
     *  where 
     *     unescape( "\\\\" + str ) = "\\" + unescape( str )
     *     unescape( "\\n" + str ) = "\n" + unescape( str )
     *     unescape( "\\"" + str ) = "\"" + unescape( str )
     *     unescape( x + str ) = "\"" + unescape( str ) for any
     *                x such that length(x)==1 and x != "\\"
     *     unescape( "" ) = ""
     *                
     *  </pre>
     *  @param stringImage the string to convert.
     */   
    static public String unescapeString( String stringImage ) {
        // Strip off the quotes:
            stringImage = stringImage.substring(1, stringImage.length()-1 ) ;
        StringBuffer buff = new StringBuffer() ;
        int i=0 ;
        while( i < stringImage.length() ) {
            char ch = stringImage.charAt( i ) ;
            switch( ch ) {
            case '\\' :
                i+=1 ;
                switch( stringImage.charAt( i ) ) {
                    case '\\' : i += 1 ;
                                buff.append('\\') ;
                    break ;
                    case '\"' : i += 1 ;
                                buff.append('\"') ;
                    break ;
                    case 'n' : i += 1 ;
                               buff.append('\n') ;
                    break ;
                    default: Assert.assertTrue( false, "bad string" ) ; }
            break ;
            default :
                i+=1 ;
                buff.append(ch) ; } }
        return buff.toString() ;
    }
    
    private void establishStandardContext() {
        /* int */ {
            SymbolTableEntry intEntry = new SymbolTableEntry(SymbolTableEntry.TYPE_ENTRY,
                                                             "int", INT_TYPE ) ;
            enterSymbol( intEntry ) ; }
        /* bool */ {
            SymbolTableEntry boolEntry = new SymbolTableEntry(SymbolTableEntry.TYPE_ENTRY,
                                                              "bool", BOOL_TYPE ) ;
            enterSymbol( boolEntry ) ; }
        /* false */ {
            CodeGenerationRule falseCGR = new CodeGenerationRule() {
                public void apply( int numArgs, Analyser analyser, CodeEmitter codeEmitter )
                throws TurtleTalkException
                {
                    analyser.check(numArgs==0, "args after false" );
                    codeEmitter.emitPush( 0 );
                    analyser.push( BOOL_TYPE ); } } ;
            
            SymbolTableEntry falseEntry = new SymbolTableEntry( SymbolTableEntry.CONSTANT_ENTRY,
                                                               "false", falseCGR ) ;
            enterSymbol( falseEntry ) ; }
        /* true */ {
            CodeGenerationRule trueCGR = new CodeGenerationRule() {
                public void apply( int numArgs, Analyser analyser, CodeEmitter codeEmitter )
                throws TurtleTalkException
                {
                    analyser.check(numArgs==0, "args after true" );
                    codeEmitter.emitPush( 1 );
                    analyser.push( BOOL_TYPE ); } } ;
            
            SymbolTableEntry trueEntry = new SymbolTableEntry( SymbolTableEntry.CONSTANT_ENTRY,
                                                               "true", trueCGR ) ;
            enterSymbol( trueEntry ) ; }
    }                                                               
}