package turtle.talk;

/** A sink for virtual machine code.
 *  <p> The routines of this interface can be used to
 *  Emit instructions for the virtual machine.
 *  <p>Abstract State:
 *  <ul>
 *    <li> code : an array of bytes </li>
 *    <li> endOfCode: an integer </li>
 *    <li> string: an array of Strings </li>
 *   </ul>
 * 
 * <p>Title: Turtle-talk</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 * @see OpCode OpCode
 */

interface CodeEmitter extends OpCode {
    
    
    /** Indicate that a new program will be emitted.
    * <p><b>Intended Function:</b>
    * <pre>
    *    [endOfCode := 0 ]
    * </pre>
    */
    public void newProgram() ;
    
    /** Emit a single byte
    * <p><b>Intended Function:</b>
    * <pre>
    *    [endOfCode < code.length --> code[endOfCode], endOfCode := b, endOfCode+1
    *    | else --> throw ResourceExhausted
    *    ]
    * </pre>
    */
    public void emitByte( byte b ) throws ResourceExhausted ;
    
    
    
    /** Emit two bytes
    * <p><b>Intended Function:</b>
    * <pre>
    * [ -(2^31) <= i < 2^31 -->
    *    [endOfCode < code.length-1 -->
    *       code[endOfCode], code[endOfCode+1], endOfCode := b0, b1, endOfCode+2
    *          where b0 is the lowest byte of i and b1 is the second lowest byte
    *          of i.
    *    | else --> throw ResourceExhausted
    *    ]
    * ]
    * </pre>
    */
    public void emitInt16( int i ) throws ResourceExhausted ;
    
    /** Emit four bytes
    * <p><b>Intended Function:</b>
    * <pre>
    *    [endOfCode <= code.length-4 -->
    *       code[endOfCode,..endOfCode+4], endOfCode := [b0, b1, b2, b3], endOfCode+4
    *          where b0 is the lowest byte of i, b1 is the second lowest byte of i,
    *            and b2 is the third lowest byte of i, and b3 is the high byte of i
    *    | else --> throw ResourceExhausted
    *    ]
    * </pre>
    */
    public void emitInt32( int i ) throws ResourceExhausted ;
    
    /** Emit a push instruction
     *  <p> There are three kinds of push instruction taking 1, 3, or 4
     *  bytes. This operation selects the shortest and emits it.
    * <p><b>Intended Function:</b>
    * Let sps(i) be the shortest push instruction to push i.
    * <pre>
    *    [endOfCode <= code.length-length(sps(i) -->
    *       code[endOfCode,...endOfCode+length(sps(i)), endOfCode
    *       := sps(i), endOfCode+length(sps(i))
    *    | else --> throw ResourceExhausted
    *    ]
    * </pre>
    */
    public void emitPush( int i )  throws ResourceExhausted ;
    
    /** Return the address of the next location not yest filled.
    * <p><b>Intended Function:</b>
    * Let sps(i) be the shortest push instruction to push i.
    * <pre>
    *    [ return endOfCode ]
    * </pre>
    */
    public int currentInstructionAddress() ;
    
    /** Change a byte already emitted.
    * <p><b>Intended Function:</b>
    * <pre>
    *    [ 0<= address < endOfCode -->
    *        code[endOfCode] := b
    *    ]
    * </pre>
    */
    public void patchByte( int address, byte b ) ;
    
    /** Change two bytes already emitted.
    * <p><b>Intended Function:</b>
    * <pre>
    * 
    * [ -(2^31) <= i < 2^31 -->
    *    [ 0<= address < endOfCode-1 -->
    *        code[endOfCode], code[endOfCode+1] := b0, b1
    *        where b0 is the lowest byte of i and
    *              b1 is the seconde lowest byte of i
    *    ]
    * ]
    * </pre>
    */
    public void patchInt16( int address, int i ) ;
    
    /** Add an entry to the string array
    * <p><b>Intended Function:</b>
    * <pre>
    * 
    * [ there is a free slot in the string array
    *   --> string[i] := str
    *       and then return i
    *       where i is an unused slot in the string array
    * |
    *   else
    *   --> throw ResourceExhausted
    * ]
    * </pre>
    */
    public int putString( String str ) throws ResourceExhausted ;
}