package turtle.talk;

/** Constants for various opcodes.
 *  Instructions in the virtual machine are of length 1,3, or 5 bytes.
 *  The majority are 1 byte long and operate on a stack.
 *  <p> The internal state of the machine consists of
 *  <ul>
 *     <li> A stack of integers (range {-(2^31) ,.. 2^31 } ) </li>
 *     <li> An array of integer memory locations: vars</li>
 *     <li> An array of strings: string
 *     <li> A program counter: pc</li>
 *  </ul>
 *  <P>To push a constant there are three choices.
 *  <ul> <li> Use the PUSH32 instruction. The constant is
 *            in the 4 subsequent bytes in signed 2's complement. </li>
 *       <li> Use the PUSH16 instruction. The constant is
 *             in the 2 subsequent bytes in signed 2's complement. </li>
 *       <li> Use a single byte instruction with a negative opcode.
 *            The constant is the negation of the opcode minus one.
 *            I.e. opcode = -1 means push(0), opcode = -2 means push(1) etc. </li>
 *  </ul>
 * <p>Title: Turtle-talk</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

interface OpCode {
// The instruction set

    /** An invalid instruction. */
    public static final byte INVALID = 0;
    
    // Operations
    
    /** y = pop() ; x = pop() ; push( x + y ) ; pc <- pc+1 */
    public static final byte ADD = 1;
    /** y = pop() ; x = pop() ; push( x - y ) ; pc <- pc+1 */
    public static final byte SUBTRACT = 2;
    /** y = pop() ; x = pop() ; push( x * y ) ; pc <- pc+1 */
    public static final byte MULTIPLY = 3;
    /** y = pop() ; x = pop() ; push( x / y ) ; pc <- pc+1
     *  <p> Note that division be 0 evaluates to 0. */
    public static final byte DIVIDE = 4;
    /** y = pop() ; x = pop() ; push( x % y ) ; pc <- pc+1 
     *  <p> Note that division be 0 evaluates to 0. */
    public static final byte MOD = 5;
    /** y = pop() ; x = pop() ; push( x == y ) ; pc <- pc+1
     *  <p> 1 represents true and 0 represents false */
    public static final byte EQUAL = 6;
    /** y = pop() ; x = pop() ; push( x < y ) ; pc <- pc+1 
     *  <p> 1 represents true and 0 represents false */
    public static final byte LESS = 7;
    /** y = pop() ; x = pop() ; push( x _< y ) ; pc <- pc+1
     *  <p> 1 represents true and 0 represents false */
    public static final byte LESSOREQUAL = 8;
    /** y = pop() ; x = pop() ; push( x or y ) ; pc <- pc+1 
     *  <p> 1 represents true and 0 represents false */
    public static final byte OR = 9;
    /** x = pop() ; push( - x ) ; pc <- pc+1 */
    public static final byte NEGATE = 10;
    /** x = pop() ; push( if x==0 then 1 else 0 ) ; pc <- pc+1 */
    public static final byte NOT = 11;
    
    // Constants
    
    
    /** PUSH16: push( the signed int formed from the next two code bytes ) ; pc <- pc+3 */
    public static final byte PUSH16=20;
    
    /** PUSH32: push (the signed int formed from the next four code bytes ) ; pc <- pc+5 */
    public static final byte PUSH32=21;
    
        // Stack manipulation.
        
    /** FLIP: y=pop() ; x=pop() ; push(y) ; push(x) */
    public static final byte FLIP=22;
    
    /** POP: y=pop() */
    public static final byte POP=23;
    
        // Variables.
        
    /** FETCH: x = pop() ; push( vars[x] ) ; pc <- pc+1 */
    public static final byte FETCH=24;
    /** STORE: y = pop() ; x = pop() ; vars[y] <- x ; pc <- pc+1 */
    public static final byte STORE=25;
    
        // Branching operations
        
    /** BRANCH: x = pop() ; pc <- x ; */
    public static final byte BRANCH = 26;
    /** FBRANCH: y = pop ; x = pop ; if( x==0 ) pc <-y else pc <- pc+1 */
    public static final byte FBRANCH = 27;
    
        // IO Instructions
        
    /** PUTINT: x = pop() ; print x ; pc <- pc+1 */
    public static final byte PUTINT = 28 ;
    
    /** PUTSTRING: x = pop() ; print strings(x) ; pc <- pc+1 */
    public static final byte PUTSTRING = 29 ;
    
    /** GETINT: push( an integer obtained from the user ) ; pc <- pc+1 */
    public static final byte GETINT = 30 ;
    
        // Maze operations. 
    
    /** GO: x = pop() ; move the turtle in the direction indicated ; pc <- pc+1 */
    public static final byte GO = 40 ;
    
    /** WALL: x = pop() ; if( there is wall in that direction ) push true ;
      *                   else push false ;
      *       pc <- pc+1 */
    public static final byte WALL = 41 ;
    
    /** GOAL: x = pop() ; if( there is the goal in that direction ) push true ;
     *                   else push false ;
     *     pc <- pc+1 */
    public static final byte GOAL = 42 ;
    
    /** SNIFF: push( the direction the goal is most in ) ; pc <- pc+1
     * In case of a tie, either direction is pushed */
    public static final byte SNIFF = 43 ;
    
    /** PUTMARK: x = pop ; mark the current square with x */
    public static final byte PUTMARK = 44 ;
    
    /** GETMARK: push( the last mark put on this square, or 0 if none ) */
    public static final byte GETMARK = 45 ;
    
    /** NEXT: x = pop() ; push( the direction clockwise from x ) ; pc <- pc+1 */
   public static final byte NEXT = 46 ;
        
        // Miscellaneous
    /** HALT stop the machine */
    public static final byte HALT = 60 ;
}