package turtle.talk;

/** An entry in the symbol table.
 *  <p> Contains all the information one might want about a
 *  a named entity in a program.
 *  <p> State
 *  <ul>
 *    <li> name : a string </li>
 *    <li> kind : an integer representing the kind of entity </li>
 *    <li> type : an integer representing the type associated with the entity </li>
 *    <li> address : an integer representing the address associated with the entity </li>
 *    <li> codeGenerationRule : a CodeGenerationRule associated with the entity </li>
 *  </ul>
 *  <p>These fields are considered absent or present according to the following table
 *  <table>
 *     <tr><th>kind</th>              <th>present</th> </tr>
 *     <tr><td>UNKNOWN_ENTRY<td>      <td>name</td></tr>
 *     <tr><td>TYPE_ENTRY<td>         <td>name, type</td></tr>
 *     <tr><td>VAR_ENTRY<td>          <td>name, type, address</td></tr>
 *     <tr><td>CONSTANT_ENTRY<td>     <td>name, codeGenenerationRule</td></tr>
 *     <tr><td>FUNCTION_ENTRY<td>     <td>name, codeGenenerationRule</td></tr>
 *     <tr><td>PROCEDURE_ENTRY<td>    <td>name, codeGenenerationRule</td></tr>
 *  </table>
 * <p>Title: Turtle-talk</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

class SymbolTableEntry {

    public static final int VAR_ENTRY = 0 ;
    public static final int CONSTANT_ENTRY = 1 ;
    public static final int PROCEDURE_ENTRY = 2 ;
    public static final int FUNCTION_ENTRY = 3 ;
    public static final int TYPE_ENTRY = 4 ;
    public static final int UNKNOWN_ENTRY = 5 ;
                            
    private int kind ;
    private int type ;
    private int address ;
    private String name ;
    private CodeGenerationRule codeGenerationRule ;
    
    public SymbolTableEntry( int kind, String name )
    {
        Assert.assertTrue( kind == UNKNOWN_ENTRY, "bad kind" ) ;
        this.kind = UNKNOWN_ENTRY ;
        this.name = name ;
    }
    
    public SymbolTableEntry( int kind, String name, int type, int address)
    {
        Assert.assertTrue( kind == VAR_ENTRY, "bad kind" ) ;
        this.kind = VAR_ENTRY ;
        this.name = name ;
        this.type = type ;
        this.address = address ;
    }
    
    public SymbolTableEntry( int kind, String name, int type)
    {
        Assert.assertTrue( kind == TYPE_ENTRY, "bad kind" ) ;
        this.kind = TYPE_ENTRY ;
        this.name = name ;
        this.type = type ;
    }
    
    public SymbolTableEntry( int kind, String name, CodeGenerationRule cgr)
    {
        Assert.assertTrue( kind == CONSTANT_ENTRY || kind == FUNCTION_ENTRY || kind == PROCEDURE_ENTRY,
                           "bad kind" ) ;
        this.kind = kind ;
        this.name = name ;
        this.codeGenerationRule = cgr ;
    }
    
    public int getKind() { return kind ; }
    
    public String getName() { return name ; }
    
    public int getAddress()
    { 
        Assert.assertTrue( kind == VAR_ENTRY, "bad kind" ) ;
        return address ;
    }
    
    public int getType()
    { 
        Assert.assertTrue( kind == VAR_ENTRY || kind==TYPE_ENTRY, "bad kind" ) ;
        return type ;
    }
    
    public CodeGenerationRule getCodeGenerationRule()
    {
        Assert.assertTrue( kind == CONSTANT_ENTRY || kind == FUNCTION_ENTRY || kind == PROCEDURE_ENTRY,
                           "bad kind" ) ;
        return codeGenerationRule ;
    }
}
